from mpl_toolkits.axes_grid1 import make_axes_locatable
import matplotlib.pyplot as plt
import numpy as np


from . import evalgeo


def print_domain(domain):
    fig, ax = plt.subplots(figsize=(6,6))
    im = ax.imshow(domain)
    cb = plt.colorbar(im)
    cb.set_label('Estimated building height [m]')
    plt.tight_layout()
    plt.savefig('domain.pdf')



def print_entropy(x,y):


    xtick_font = {
        "family": "DejaVu Sans",
        "size": 10,
        "weight": "bold",
        "alpha": 1.0,
        "zorder": 3,
    }

    color="#003366"
    edgecolor="k"
    linewidth=0.5
    alpha=0.7

    #dir_topo = '/Users/jiachenlu/forSimulationOnly/highRes/'

    fig, ax = plt.subplots(figsize=(6,6), subplot_kw={"projection": "polar"})

    ax.set_theta_zero_location("N") # 
    ax.set_theta_direction("clockwise")
    ax.set_ylim(top=y.max())

    # configure the y-ticks and remove their labels
    ax.set_yticks(np.linspace(0, y.max(), 5))
    ax.set_yticklabels(labels="")

    # configure the x-ticks and their labels
    xticklabels = ["N", "", "E", "", "S", "", "W", ""]
    ax.set_xticks(ax.get_xticks())
    ax.set_xticklabels(labels=xticklabels, fontdict=xtick_font)
    ax.tick_params(axis="x", which="major", pad=-2)

    ax.bar(
        x,
        height=y,
        width=0.1,
        align="center",
        bottom=0,
        zorder=2,
        color=color,
        edgecolor=edgecolor,
        linewidth=linewidth,
        alpha=alpha,
    )
    ax.set_thetamin(-90)
    ax.set_thetamax(90)
    plt.tight_layout()
    plt.savefig('polarO.png',dpi=300)
    
    dist = 2 # Edge evenly distributed to just 2 orientations
    hMin = 1/dist*np.log(1/dist)*dist

    dist = 18 # Edge evenly distributed to all 18 orientations
    hMax = 1/dist*np.log(1/dist)*dist

    H_w = np.nansum(y/sum(y)*1*np.log(y/sum(y)*1))
    phi = 1-(hMax-H_w)/(hMax-hMin)
    
    return(phi)

def print_results(phi,domain,area,areaH,bldH):
    print('$\phi=$'+str(phi))
    print(domain.shape)
    print('Rasterization error = '+str((evalgeo.cal_lp(domain)-np.array(area).sum()/domain.size)/evalgeo.cal_lp(domain)*100)[:4]+'%')
    print('{}%  real building height recognized, the rest is set to {} m'.format(str(np.array(areaH).sum()/np.array(area).sum()*100)[:4],bldH))
    return()



def showDiagram(domain,angle,weighted,phi,angleRotate,area,name):
        # Triming for proper size and fit the computational request for parallelization
    # 300,330
    dir_topo = ''
    fig = plt.figure(figsize=(12, 6))


    ax1 = fig.add_subplot(1,3,1)
    ax2 = fig.add_subplot(1,3,2)

    ax1.contourf(domain)
    ax1.axis('equal')


    lp = evalgeo.cal_lp(domain)
    s = (1-lp)*0.001

    ax1.set_title('Computaional domain ' +'$\\lambda_p$ = '+str(lp)[0:6]+'\n'+ name + ' Domain size = '
                  + str(domain.shape))

    np.savetxt(dir_topo+name+'_topo',domain,fmt='%d')



    ax2.contour(domain,linewidths=0.1,colors='r')    
    ax2.axis('equal')
    ax2.set_title('Top sink of scalar = 1e-7*'+str(s)[5:9])


    ax31= fig.add_subplot(133, polar=True)



    xtick_font = {
        "family": "DejaVu Sans",
        "size": 15,
        "weight": "bold",
        "alpha": 1.0,
        "zorder": 3,
    }

    color="#003366"
    edgecolor="k"
    linewidth=0.5
    alpha=0.7


    x = np.unique(angle)-angleRotate
    

    x = np.deg2rad(x)
    y = np.array(weighted)
    
    x = np.concatenate((x,x+np.pi),axis=0)
    y = np.concatenate((y,y),axis=0)


    ax31.set_theta_zero_location("N")
    ax31.set_theta_direction("clockwise")
    ax31.set_ylim(top=y.max())

    # configure the y-ticks and remove their labels
    ax31.set_yticks(np.linspace(0, y.max(), 5))
    ax31.set_yticklabels(labels="")

    
    
    # configure the x-ticks and their labels
    xticklabels = ["N", "", "E", "", "S", "", "W", ""]
    ax31.set_xticks(ax31.get_xticks())
    ax31.set_xticklabels(labels=xticklabels, fontdict=xtick_font)
    ax31.tick_params(axis="x", which="major", pad=-2)

    ax31.bar(
        x,
        height=y,
        width=0.1,
        align="center",
        bottom=0,
        zorder=2,
        color=color,
        edgecolor=edgecolor,
        linewidth=linewidth,
        alpha=alpha,
    )
    
    #ax31.set_thetamin(-90)
    #ax31.set_thetamax(90)
    
    
    gamma,prof = evalgeo.cal_alignness(domain)
    

    ax32= fig.add_subplot(8,3,24)
    title = 'Orientations of building edges \n Edge entropy $\phi$  = '+ str(phi)[:6]
    title += '\n Rotation angle  = '+str(angleRotate)+'$^{\circ}$'
    title += '\n Average building size $A_0$  = '+str(area.mean())[:6]+'$m^2$'
    ax31.set_title(title)


    ax32.plot(prof,c='k')
    ax32.set_xlim(0,domain.shape[0])
    #ax32.frame('off')
    ax32.set_xlabel('Alignedness $\gamma$ = '+str(gamma)[0:6])


    plt.tight_layout()
    plt.savefig(dir_topo + name + '.png',dpi=300)



import numpy as np
import pandas as pd
from shapely.geometry import Point,Polygon


from . import entropy

def initialize(bbox,df,x_mul,y_mul,bbox_cor):
    d = {}
    i = 0
    x_min = bbox_cor[1]
    y_min = bbox_cor[2]
    # gdf to df
    for idxP in range(df.shape[0]):
        try:
            d[i] = {'geometry': df['geometry'][idxP].exterior.coords.xy, 'cent': df['cent'][idxP].coords.xy}
            i+=1
        except:
            i = i
    df = pd.DataFrame.from_dict(d, "index")
    # Convert lat/long to x/y 2D empty domain
    bb = np.zeros(np.shape(bbox))
    xxx = []
    yyy = []
    for i in range(np.shape(bbox)[0]):
        bb[i][0] = int(np.round((bbox[i][0] - x_min)*x_mul))
        bb[i][1] = int(np.round((bbox[i][1] - y_min)*y_mul))
        xxx.append(int(np.round((bbox[i][0] - x_min)*x_mul)))
        yyy.append(int(np.round((bbox[i][1] - y_min)*y_mul)))
        
    domain = np.zeros([np.max(xxx)-np.min(xxx),np.max(yyy)-np.min(yyy)])
    return(domain,df,np.min(xxx),np.min(yyy))



def process_building(lineO,lineL,domain,idxP,gdf,bHeightL,bHeightH,x_mul,y_mul,x_start,y_start,bbox_cor,bldH,area,areaH,res):
    # poly.bounds
    nx = []
    ny = []
    xy = []
    Pol = []
    # Collect and project points in the geometry
    for i in range(np.shape(gdf['geometry'][idxP])[1]):
        xx = int(np.round((gdf['geometry'][idxP][0][i] - bbox_cor[1])*x_mul))
        yy = int(np.round((gdf['geometry'][idxP][1][i] - bbox_cor[2])*y_mul))
        nx.append(xx)
        ny.append(yy)
        xy.append([xx,yy])
        Pol.append((xx-x_start,yy-y_start))
        
    [lineO,lineL] = entropy.count_edge(xy,lineO,lineL,res) # working

    pol = Polygon(Pol)
    area.append(pol.area)
    AA = True

   
        
    # transfer all nan to zeros first

    bHeightL[np.isnan(bHeightL)] = 0
    bHeightH[np.isnan(bHeightH)] = 0

    if bHeightH[idxP]==0: # consider the actual height first
        if bHeightL[idxP]==0: 
            HH = bldH  # no bHeightL and bHeightH
            AA = False

        else:
            #print('L')
            HH = float(bHeightL[idxP])
            areaH.append(pol.area) # count surface of buildings that have real heights
    else:
        #print('H')
        HH = float(bHeightH[idxP])
        areaH.append(pol.area)
    
    domain = construct(domain,HH,nx,ny,x_start,y_start,pol)
    
    
    return(domain,areaH,area)




def construct(domain,HH,nx,ny,x_start,y_start,pol):
    # Construct edge
    xmin = np.min(nx) - x_start
    xmax = np.max(nx) - x_start
    ymin = np.min(ny) - y_start
    ymax = np.max(ny) - y_start
    for ii in range(np.size(nx)):
        try:
            domain[nx[ii]-x_start,ny[ii]-y_start] = HH # Fill the outline with building height
        except:
            continue

    # Construct interior by filling grids with building height
    for x in range (xmin,xmax):
        for y in range(ymin,ymax):
            p = Point(x,y)
            if p.within(pol) or p.intersects(pol):
                try:
                    domain[x,y] = HH  # Fill inside with building height
                except:
                    continue   
    return(domain)


import numpy as np
import math



def count_edge(xy,lineO,lineL,res):
    lineO = lineO
    lineL = lineL
    for line in range(np.shape(xy)[0]-1):
        lineO.append(np.rad2deg(np.arctan((xy[line][1]-xy[line+1][1])/(xy[line][0]-xy[line+1][0]+1e-10))))
        lineL.append(math.dist(xy[line], xy[line+1])*res)
    return(lineO,lineL)
        
        
def cal_entropy(lineO,lineL):
    angle = np.zeros(np.shape(lineO)[0])
    
    for i in range(np.shape(lineO)[0]):
        angle[i] = np.round(lineO[i]/10)*10

    x,y=zip(*sorted(zip(angle.flatten(),np.array(lineL).flatten())))

    weighted = np.zeros(np.unique(angle).size)
    i = 0
    idx = 0

    while i < angle.size-1:
        if x[i] == x[i+1]:
            weighted[idx] = weighted[idx] + y[i]
            i += 1
        else:

            weighted[idx] = weighted[idx] + y[i]
            i += 1
            idx += 1
    weighted[idx] += y[-1]
    angle = np.unique(angle)
    
    if (angle[0] == -90) and (angle[-1] == 90):
        print('Merged two angle data')
    #weighted.size == 19: # in case 19 directions are recorded
        weighted[-1] += weighted[0]
        weighted = np.delete(weighted, 0)
        angle = np.delete(angle, 0)


    
    ################
    x = np.deg2rad(angle)
    y = weighted
    #################
    
    
    return(x,y)





from setuptools import setup,find_packages
setup(name='OSM2LES',
      version='0.0.12',
      description='Transfer building footprint to 2D DEM for LES simulation',
      author='Jiachen Lu',
      author_email='jiachensc@gmail.com',
      requires= ['numpy','matplotlib','shapely','pandas','scipy','osmnx'], 
      install_requires= ['numpy','matplotlib','shapely','pandas','scipy','osmnx','netCDF4'], 
      packages=["OSM2LES"],
      license="MIT",
      python_requires=">=3.6",
      )





xtick_font = {
    "family": "DejaVu Sans",
    "size": 10,
    "weight": "bold",
    "alpha": 1.0,
    "zorder": 3,
}

color="#003366"
edgecolor="k"
linewidth=0.5
alpha=0.7

#dir_topo = '/Users/jiachenlu/forSimulationOnly/highRes/'



